@@ -1,13 +1,21 @@
 import urllib, threading, re, logging, socket, sys, base64
 from netlib import tcp, http, odict, wsgi
 import netlib.utils
-import version, app, language
+import version, app, language, utils
 
 logger = logging.getLogger('pathod')
 
 class PathodError(Exception): pass
 
 
+class SSLOptions:
+    def __init__(self, certfile=None, keyfile=None, not_after_connect=None, request_client_cert=False):
+        self.keyfile = keyfile or utils.data.path("resources/server.key")
+        self.certfile = certfile or utils.data.path("resources/server.crt")
+        self.not_after_connect = not_after_connect
+        self.request_client_cert = request_client_cert
+
+
 class PathodHandler(tcp.BaseHandler):
     wbufsize = 0
     sni = None
@@ -20,7 +28,7 @@ class PathodHandler(tcp.BaseHandler):
     def serve_crafted(self, crafted, request_log):
         c = self.server.check_policy(crafted, self.server.request_settings)
         if c:
-            err = language.PathodErrorResponse(c)
+            err = language.make_error_response(c)
             language.serve(err, self.wfile, self.server.request_settings)
             log = dict(
                 type = "error",
@@ -28,8 +36,9 @@ class PathodHandler(tcp.BaseHandler):
             )
             return False, log
 
-        if self.server.explain:
+        if self.server.explain and not isinstance(crafted, language.PathodErrorResponse):
             crafted = crafted.freeze(self.server.request_settings, None)
+            self.info(">> Spec: %s"%crafted.spec())
         response_log = language.serve(crafted, self.wfile, self.server.request_settings, None)
         log = dict(
                 type = "crafted",
@@ -54,19 +63,54 @@ class PathodHandler(tcp.BaseHandler):
             # Normal termination
             return False, None
 
-        parts = http.parse_init_http(line)
-        if not parts:
+        m = utils.MemBool()
+        if m(http.parse_init_connect(line)):
+            headers = http.read_headers(self.rfile)
+            self.wfile.write(
+                        'HTTP/1.1 200 Connection established\r\n' +
+                        ('Proxy-agent: %s\r\n'%version.NAMEVERSION) +
+                        '\r\n'
+                        )
+            self.wfile.flush()
+            if not self.server.ssloptions.not_after_connect:
+                try:
+                    self.convert_to_ssl(
+                        self.server.ssloptions.certfile,
+                        self.server.ssloptions.keyfile,
+                        handle_sni = self.handle_sni,
+                        request_client_cert = self.server.ssloptions.request_client_cert
+                    )
+                except tcp.NetLibError, v:
+                    s = str(v)
+                    self.info(s)
+                    return False, dict(type = "error", msg = s)
+            return True, None
+        elif m(http.parse_init_proxy(line)):
+            method, _, _, _, path, httpversion = m.v
+        elif m(http.parse_init_http(line)):
+            method, path, httpversion = m.v
+        else:
             s = "Invalid first line: %s"%repr(line)
             self.info(s)
             return False, dict(type = "error", msg = s)
 
-        method, path, httpversion = parts
         headers = http.read_headers(self.rfile)
         if headers is None:
             s = "Invalid headers"
             self.info(s)
             return False, dict(type = "error", msg = s)
 
+        clientcert = None
+        if self.clientcert:
+            clientcert = dict(
+                cn = self.clientcert.cn,
+                subject = self.clientcert.subject,
+                serial = self.clientcert.serial,
+                notbefore = self.clientcert.notbefore.isoformat(),
+                notafter = self.clientcert.notafter.isoformat(),
+                keyinfo = self.clientcert.keyinfo,
+            )
+
         request_log = dict(
             path = path,
             method = method,
@@ -74,6 +118,7 @@ class PathodHandler(tcp.BaseHandler):
             httpversion = httpversion,
             sni = self.sni,
             remote_address = self.client_address,
+            clientcert = clientcert
         )
 
         try:
@@ -98,13 +143,13 @@ class PathodHandler(tcp.BaseHandler):
                 crafted = language.parse_response(self.server.request_settings, spec)
             except language.ParseException, v:
                 self.info("Parse error: %s"%v.msg)
-                crafted = language.PathodErrorResponse(
+                crafted = language.make_error_response(
                         "Parse Error",
                         "Error parsing response spec: %s\n"%v.msg + v.marked()
                     )
             return self.serve_crafted(crafted, request_log)
         elif self.server.noweb:
-            crafted = language.PathodErrorResponse("Access Denied")
+            crafted = language.make_error_response("Access Denied")
             language.serve(crafted, self.wfile, self.server.request_settings)
             return False, dict(type = "error", msg="Access denied: web interface disabled")
         else:
@@ -133,11 +178,13 @@ class PathodHandler(tcp.BaseHandler):
         self.info("\n".join(s))
 
     def handle(self):
-        if self.server.ssloptions:
+        if self.server.ssl:
             try:
                 self.convert_to_ssl(
-                    self.server.ssloptions["certfile"],
-                    self.server.ssloptions["keyfile"],
+                    self.server.ssloptions.certfile,
+                    self.server.ssloptions.keyfile,
+                    handle_sni = self.handle_sni,
+                    request_client_cert = self.server.ssloptions.request_client_cert
                 )
             except tcp.NetLibError, v:
                 s = str(v)
@@ -171,14 +218,14 @@ class PathodHandler(tcp.BaseHandler):
 class Pathod(tcp.TCPServer):
     LOGBUF = 500
     def __init__(   self,
-                    addr, ssloptions=None, craftanchor="/p/", staticdir=None, anchors=None,
+                    addr, ssl=False, ssloptions=None, craftanchor="/p/", staticdir=None, anchors=None,
                     sizelimit=None, noweb=False, nocraft=False, noapi=False, nohang=False,
                     timeout=None, logreq=False, logresp=False, explain=False, hexdump=False
                 ):
         """
             addr: (address, port) tuple. If port is 0, a free port will be
             automatically chosen.
-            ssloptions: a dictionary containing certfile and keyfile specifications.
+            ssloptions: an SSLOptions object.
             craftanchor: string specifying the path under which to anchor response generation.
             staticdir: path to a directory of static resources, or None.
             anchors: A list of (regex, spec) tuples, or None.
@@ -188,7 +235,8 @@ class Pathod(tcp.TCPServer):
             nohang: Disable pauses.
         """
         tcp.TCPServer.__init__(self, addr)
-        self.ssloptions = ssloptions
+        self.ssl = ssl
+        self.ssloptions = ssloptions or SSLOptions()
         self.staticdir = staticdir
         self.craftanchor = craftanchor
         self.sizelimit = sizelimit
@@ -196,9 +244,7 @@ class Pathod(tcp.TCPServer):
         self.timeout, self.logreq, self.logresp, self.hexdump = timeout, logreq, logresp, hexdump
         self.explain = explain
 
-        if not noapi:
-            app.api()
-        self.app = app.app
+        self.app = app.make_app(noapi)
         self.app.config["pathod"] = self
         self.log = []
         self.logid = 0
